;------------------------------------------------------------------------------
; C8051F930.INC
;------------------------------------------------------------------------------
; Copyright 2007 Silicon Laboratories, Inc.
; http://www.silabs.com
;
; Program Description:
;
; Register/bit definitions for the C8051F93x family.
;
;
; Target:         C8051F930, 'F931, 'F920, 'F921
; Tool chain:     Keil
; Command Line:   None
;
; Release 1.3
;    - Corrected address of CRC0FLIP
;    - 28 JUL 2009 (FB)
;
; Release 1.2
;    - Added definitions for ONESHOT and FLSCL.
;    - 4 OCT 2007 (FB)
;
; Release 1.1
;    - Switched the addresses of DC0CF and DC0CN.
;    - 20 SEP 2007 (FB)
;
; Release 1.0
;    -Initial Revision (FB)
;    -18 SEP 2007
;

;------------------------------------------------------------------------------
; Byte Registers
;------------------------------------------------------------------------------

P0          DATA 080H                      ; Port 0 Latch
SP          DATA 081H                      ; Stack Pointer
DPL         DATA 082H                      ; Data Pointer Low
DPH         DATA 083H                      ; Data Pointer High
CRC0CN      DATA 084H                      ; CRC0 Control
CRC0IN      DATA 085H                      ; CRC0 Input
CRC0DAT     DATA 086H                      ; CRC0 Data
PCON        DATA 087H                      ; Power Control
TCON        DATA 088H                      ; Timer/Counter Control
TMOD        DATA 089H                      ; Timer/Counter Mode
TL0         DATA 08AH                      ; Timer/Counter 0 Low
TL1         DATA 08BH                      ; Timer/Counter 1 Low
TH0         DATA 08CH                      ; Timer/Counter 0 High
TH1         DATA 08DH                      ; Timer/Counter 1 High
TOFFL       DATA 08DH                      ; Temperature Offset Low
TOFFH       DATA 08EH                      ; Temperature Offset High
CKCON       DATA 08EH                      ; Clock Control
PSCTL       DATA 08FH                      ; Program Store R/W Control
P1          DATA 090H                      ; Port 1 Latch
TMR3CN      DATA 091H                      ; Timer/Counter 3 Control
TMR3RLL     DATA 092H                      ; Timer/Counter 3 Reload Low
TMR3RLH     DATA 093H                      ; Timer/Counter 3 Reload High
TMR3L       DATA 094H                      ; Timer/Counter 3 Low
TMR3H       DATA 095H                      ; Timer/Counter 3 High
ADC0MX      DATA 096H                      ; AMUX0 Channel Select
ADC0CF      DATA 097H                      ; ADC0 Configuration
SCON0       DATA 098H                      ; UART0 Control
SBUF0       DATA 099H                      ; UART0 Data Buffer
P0DRV       DATA 099H                      ; Port 0 Drive Strength
CRC0CNT     DATA 09AH                      ; CRC0 Automatic Flash Sector Count
CPT0CN      DATA 09BH                      ; Comparator0 Control
P1DRV       DATA 09BH                      ; Port 1 Drive Strength
CRC0FLIP    DATA 09CH                      ; CRC0 Flip
CPT0MD      DATA 09DH                      ; Comparator0 Mode Selection
P2DRV       DATA 09DH                      ; Port 2 Drive Strength
CRC0AUTO    DATA 09EH                      ; CRC0 Automatic Control
CPT0MX      DATA 09FH                      ; Comparator0 Mux Selection
P2          DATA 0A0H                      ; Port 2 Latch
SPI0CFG     DATA 0A1H                      ; SPI0 Configuration
SPI0CKR     DATA 0A2H                      ; SPI0 Clock Rate Control
SPI0DAT     DATA 0A3H                      ; SPI0 Data
P0MDOUT     DATA 0A4H                      ; Port 0 Output Mode Configuration
P1MDOUT     DATA 0A5H                      ; Port 1 Output Mode Configuration
P2MDOUT     DATA 0A6H                      ; Port 2 Output Mode Configuration
SFRPAGE     DATA 0A7H                      ; SFR Page
IE          DATA 0A8H                      ; Interrupt Enable
CLKSEL      DATA 0A9H                      ; Clock Select
CS0CF 	    DATA 0AAH                      ; CS0 Configuration
CS0MX       DATA 0ABH                      ; CS0 Mux Channel Select
RTC0ADR     DATA 0ACH                      ; RTC0 Address
RTC0DAT     DATA 0ADH                      ; RTC0 Data
RTC0KEY     DATA 0AEH                      ; RTC0 Key
CS0MD1      DATA 0AFH                      ; CS0 Mode 1
CS0CN       DATA 0B0H                      ; CS0 Control
OSCXCN      DATA 0B1H                      ; External Oscillator Control
OSCICN      DATA 0B2H                      ; Internal Oscillator Control
OSCICL      DATA 0B3H                      ; Internal Oscillator Calibration
PMU0CF      DATA 0B5H                      ; PMU0 Configuration
PMU0MD      DATA 0B5H                      ; PMU0 Mode
FLSCL       DATA 0B6H                      ; Flash Scale Register
FLKEY       DATA 0B7H                      ; Flash Lock And Key
IP          DATA 0B8H                      ; Interrupt Priority
IREF0CF     DATA 0B9H                      ; Current Reference IREF0 Configuration
ADC0AC      DATA 0BAH                      ; ADC0 Accumulator Configuration
ADC0PWR     DATA 0BBH                      ; ADC0 Burst Mode Power-Up Time
ADC0TK      DATA 0BCH                      ; ADC0 Tracking Control
ADC0L       DATA 0BDH                      ; ADC0 Low
ADC0H       DATA 0BEH                      ; ADC0 High
P1MASK      DATA 0BFH                      ; Port 1 Mask
SMB0CN      DATA 0C0H                      ; SMBus0 Control
SMB0CF      DATA 0C1H                      ; SMBus0 Configuration
SMB0DAT     DATA 0C2H                      ; SMBus0 Data
ADC0GTL     DATA 0C3H                      ; ADC0 Greater-Than Compare Low
ADC0GTH     DATA 0C4H                      ; ADC0 Greater-Than Compare High
ADC0LTL     DATA 0C5H                      ; ADC0 Less-Than Compare Word Low
ADC0LTH     DATA 0C6H                      ; ADC0 Less-Than Compare Word High
P0MASK      DATA 0C7H                      ; Port 0 Mask
TMR2CN      DATA 0C8H                      ; Timer/Counter 2 Control
REG0CN      DATA 0C9H                      ; Voltage Regulator (REG0) Control
TMR2RLL     DATA 0CAH                      ; Timer/Counter 2 Reload Low
TMR2RLH     DATA 0CBH                      ; Timer/Counter 2 Reload High
TMR2L       DATA 0CCH                      ; Timer/Counter 2 Low
TMR2H       DATA 0CDH                      ; Timer/Counter 2 High
PMU0FL      DATA 0CEH                      ; PMU0 Flag Register
P1MAT       DATA 0CFH                      ; Port 1 Match
PSW         DATA 0D0H                      ; Program Status Word
REF0CN      DATA 0D1H                      ; Voltage Reference Control
CS0SCAN0    DATA 0D2H                      ; CS0 Scan Channel Enable 0
CS0SCAN1    DATA 0D3H                      ; CS0 Scan Channel Enable 1
P0SKIP      DATA 0D4H                      ; Port 0 Skip
P1SKIP      DATA 0D5H                      ; Port 1 Skip
IREF0CN     DATA 0D6H                      ; Current Reference IREF0 Control
P0MAT       DATA 0D7H                      ; Port 0 Match
PCA0CN      DATA 0D8H                      ; PCA0 Control
PCA0MD      DATA 0D9H                      ; PCA0 Mode
PCA0CPM0    DATA 0DAH                      ; PCA0 Module 0 Mode Register
PCA0CPM1    DATA 0DBH                      ; PCA0 Module 1 Mode Register
PCA0CPM2    DATA 0DCH                      ; PCA0 Module 2 Mode Register
CS0SS       DATA 0DDH                      ; CS0 Auto-Scan Start Channel
CS0SE       DATA 0DEH                      ; CS0 Auto-Scan End Channel
CS0PM       DATA 0DEH                      ; CS0 Power Management
PCA0PWM     DATA 0DFH                      ; PCA0 PWM Configuration
ACC         DATA 0E0H                      ; Accumulator
XBR0        DATA 0E1H                      ; Port I/O Crossbar Control 0
XBR1        DATA 0E2H                      ; Port I/O Crossbar Control 1
REVID       DATA 0E2H                      ; Revision ID
XBR2        DATA 0E3H                      ; Port I/O Crossbar Control 2
DEVICEID    DATA 0E3H                      ; Device ID
IT01CF      DATA 0E4H                      ; INT0/INT1 Configuration
FLWR        DATA 0E5H                      ; Flash Write Only Register
EIE1        DATA 0E6H                      ; Extended Interrupt Enable 1
EIE2        DATA 0E7H                      ; Extended Interrupt Enable 2
ADC0CN      DATA 0E8H                      ; ADC0 Control
PCA0CPL1    DATA 0E9H                      ; PCA0 Capture 1 Low
PCA0CPH1    DATA 0EAH                      ; PCA0 Capture 1 High
PCA0CPL2    DATA 0EBH                      ; PCA0 Capture 2 Low
PCA0CPH2    DATA 0ECH                      ; PCA0 Capture 2 High
CS0DL       DATA 0EDH                      ; CS0 Data Low Byte
CS0DH       DATA 0EEH                      ; CS0 Data High Byte
RSTSRC      DATA 0EFH                      ; Reset Source Configuration/Status
B           DATA 0F0H                      ; B Register
P0MDIN      DATA 0F1H                      ; Port 0 Input Mode Configuration
P1MDIN      DATA 0F2H                      ; Port 1 Input Mode Configuration
CS0MD2      DATA 0F3H                      ; CS0 Mode 2
CS0MD3      DATA 0F3H                      ; CS0 Mode 3
SMB0ADR     DATA 0F4H                      ; SMBus Slave Address
SMB0ADM     DATA 0F5H                      ; SMBus Slave Address Mask
EIP1        DATA 0F6H                      ; Extended Interrupt Priority 1
EIP2        DATA 0F7H                      ; Extended Interrupt Priority 2
SPI0CN      DATA 0F8H                      ; SPI0 Control
PCA0L       DATA 0F9H                      ; PCA0 Counter Low
PCA0H       DATA 0FAH                      ; PCA0 Counter High
PCA0CPL0    DATA 0FBH                      ; PCA0 Capture 0 Low
PCA0CPH0    DATA 0FCH                      ; PCA0 Capture 0 High
CS0THL      DATA 0FDH                      ; CS0 Comparator Threshold Low Byte
CS0THH      DATA 0FEH                      ; CS0 Comparator Threshold High Byte
VDM0CN      DATA 0FFH                      ; VDD Monitor Control




;------------------------------------------------------------------------------
; Bit Definitions
;------------------------------------------------------------------------------

; TCON  088H
TF1     BIT     TCON.7                 ; Timer1 overflow flag
TR1     BIT     TCON.6                 ; Timer1 on/off control
TF0     BIT     TCON.5                 ; Timer0 overflow flag
TR0     BIT     TCON.4                 ; Timer0 on/off control
IE1     BIT     TCON.3                 ; Ext. Interrupt 1 edge flag
IT1     BIT     TCON.2                 ; Ext. Interrupt 1 type
IE0     BIT     TCON.1                 ; Ext. Interrupt 0 edge flag
IT0     BIT     TCON.0                 ; Ext. Interrupt 0 type

; SCON0  098H
S0MODE  BIT     SCON0.7                ; Serial mode control bit 0
                                       ; Bit 6 unused
MCE0    BIT     SCON0.5                ; Multiprocessor communication enable
REN0    BIT     SCON0.4                ; Receive enable
TB80    BIT     SCON0.3                ; Transmit bit 8
RB80    BIT     SCON0.2                ; Receive bit 8
TI0     BIT     SCON0.1                ; Transmit interrupt flag
RI0     BIT     SCON0.0                ; Receive interrupt flag

; IE  0A8H
EA      BIT     IE.7                   ; Global interrupt enable
ESPI0   BIT     IE.6                   ; SPI0 Interrupt Enable
ET2     BIT     IE.5                   ; Timer 2 Interrupt Enable
ES0     BIT     IE.4                   ; UART0 interrupt enable
ET1     BIT     IE.3                   ; Timer1 interrupt enable
EX1     BIT     IE.2                   ; External interrupt 1 enable
ET0     BIT     IE.1                   ; Timer0 interrupt enable
EX0     BIT     IE.0                   ; External interrupt 0 enable


; CS0CN 0B0H
CS0EN   BIT     CS0CN.7                ; CS0 Enable
CS0EOS  BIT     CS0CN.6                ; CS0 End Of Scan Interrupt Flag
CS0INT  BIT     CS0CN.5                ; CS0 End Of Conversion Interrupt Flag
CS0BUSY BIT     CS0CN.4                ; CS0 Busy Bit
CS0CMPEN BIT    CS0CN.3                ; CS0 Digital Comparator Enable
CS0BBB  BIT     CS0CN.2                ; CS0 BBB
CS0AAA  BIT     CS0CN.1                ; CS0 AAA
CS0CMPF BIT     CS0CN.0                ; CS0 Enable


; IP  0B8H
                                       ; Bit 7 unused
PSPI0   BIT     IP.6                   ; SPI0 interrupt priority
PT2     BIT     IP.5                   ; Timer2 priority
PS0     BIT     IP.4                   ; UART0 priority
PT1     BIT     IP.3                   ; Timer1 priority
PX1     BIT     IP.2                   ; External interrupt 1 priority
PT0     BIT     IP.1                   ; Timer0 priority
PX0     BIT     IP.0                   ; External interrupt 0 priority

; SMB0CN  0C0H
MASTER  BIT     SMB0CN.7               ; Master/slave indicator
TXMODE  BIT     SMB0CN.6               ; Transmit mode indicator
STA     BIT     SMB0CN.5               ; Start flag
STO     BIT     SMB0CN.4               ; Stop flag
ACKRQ   BIT     SMB0CN.3               ; Acknowledge request
ARBLOST BIT     SMB0CN.2               ; Arbitration lost indicator
ACK     BIT     SMB0CN.1               ; Acknowledge flag
SI      BIT     SMB0CN.0               ; SMBus interrupt flag

; TMR2CN  0C8H
TF2H    BIT     TMR2CN.7               ; Timer2 high byte overflow flag
TF2L    BIT     TMR2CN.6               ; Timer2 low byte overflow flag
TF2LEN  BIT     TMR2CN.5               ; Timer2 low byte interrupt enable
T2CE    BIT     TMR2CN.4               ; Timer2 capture enable
T2SPLIT BIT     TMR2CN.3               ; Timer2 split mode enable
TR2     BIT     TMR2CN.2               ; Timer2 on/off control
T2CSS   BIT     TMR2CN.1               ; Timer 2 Capture Source select
T2XCLK  BIT     TMR2CN.0               ; Timer2 external clock select

; PSW  0D0H
CY      BIT     PSW.7                  ; Carry flag
AC      BIT     PSW.6                  ; Auxiliary carry flag
F0      BIT     PSW.5                  ; User flag 0
RS1     BIT     PSW.4                  ; Register bank select 1
RS0     BIT     PSW.3                  ; Register bank select 0
OV      BIT     PSW.2                  ; Overflow flag
F1      BIT     PSW.1                  ; User flag 1
P       BIT     PSW.0                  ; Accumulator parity flag

; PCA0CN  0D8H
CF      BIT     PCA0CN.7               ; PCA0 counter overflow flag
CR      BIT     PCA0CN.6               ; PCA0 counter run control
CCF5    BIT     PCA0CN.5               ; PCA0 Module5 capture/compare flag
CCF4    BIT     PCA0CN.4               ; PCA0 module4 capture/compare flag
CCF3    BIT     PCA0CN.3               ; PCA0 module3 capture/compare flag
CCF2    BIT     PCA0CN.2               ; PCA0 module2 capture/compare flag
CCF1    BIT     PCA0CN.1               ; PCA0 module1 capture/compare flag
CCF0    BIT     PCA0CN.0               ; PCA0 module0 capture/compare flag

; ADC0CN  0E8H
AD0EN   BIT     ADC0CN.7               ; ADC0 Enable
BURSTEN BIT     ADC0CN.6               ; ADC0 Burst Enable
AD0INT  BIT     ADC0CN.5               ; ADC0 EOC Interrupt Flag
AD0BUSY BIT     ADC0CN.4               ; ADC0 Busy Flag
AD0WINT BIT     ADC0CN.3               ; ADC0 Window Interrupt Flag
AD0CM2  BIT     ADC0CN.2               ; ADC0 Convert Start Mode Bit 2
AD0CM1  BIT     ADC0CN.1               ; ADC0 Convert Start Mode Bit 1
AD0CM0  BIT     ADC0CN.0               ; ADC0 Convert Start Mode Bit 0

; SPI0CN  0F8H
SPIF0    BIT    SPI0CN.7               ; SPI0 interrupt flag
WCOL0    BIT    SPI0CN.6               ; SPI0 write collision flag
MODF0    BIT    SPI0CN.5               ; SPI0 mode fault flag
RXOVRN0  BIT    SPI0CN.4               ; SPI0 rx overrun flag
NSS0MD1  BIT    SPI0CN.3               ; SPI0 slave select mode 1
NSS0MD0  BIT    SPI0CN.2               ; SPI0 slave select mode 0
TXBMT0   BIT    SPI0CN.1               ; SPI0 transmit buffer empty
SPI0EN   BIT    SPI0CN.0               ; SPI0 SPI enable


; SmaRTClock Internal Registers
CAPTURE0	EQU 0x00               ; RTC address of CAPTURE0 register
CAPTURE1  	EQU 0x01               ; RTC address of CAPTURE1 register
CAPTURE2  	EQU 0x02               ; RTC address of CAPTURE2 register
CAPTURE3  	EQU 0x03               ; RTC address of CAPTURE3 register
RTC0CN    	EQU 0x04               ; RTC address of RTC0CN register
RTC0XCN   	EQU 0x05               ; RTC address of RTC0XCN register
RTC0XCF   	EQU 0x06               ; RTC address of RTC0XCF register
RTC0PIN   	EQU 0x07               ; RTC address of RTC0PIN register
ALARM0    	EQU 0x08               ; RTC address of ALARM0 register
ALARM1    	EQU 0x09               ; RTC address of ALARM1 register
ALARM2    	EQU 0x0A               ; RTC address of ALARM2 register
ALARM3    	EQU 0x0B               ; RTC address of ALARM3 register

; SmaRTClock Bit Definitions
RTC0CAP		EQU 0x01
RTC0SET   	EQU 0x02
ALRM      	EQU 0x04
RTC0AEN   	EQU 0x08
RTC0TR    	EQU 0x10
OSCFAIL   	EQU 0x20
MCLKEN    	EQU 0x40
RTC0EN    	EQU 0x80

;--------------------------------------------------------------------------------------
; Power Management Bit Definitions
SLEEP        	EQU 0x80              ; Sleep Mode Select
SUSPEND      	EQU 0x40              ; Suspend Mode Select
CLEAR        	EQU 0x20              ; Wake-Up Flag Clear
RSTWK        	EQU 0x10              ; Reset Pin Falling Edge Wake-Up
RTCFWK       	EQU 0x08              ; SmaRTClock Failure Wake-Up
RTCAWK       	EQU 0x04              ; SmaRTClock Alarm Wake-Up
PMATWK       	EQU 0x02              ; Port Match Wake-Up
CPT0WK       	EQU 0x01              ; Comparator0 Wake-Up
CS0WK		EQU 0x01	      ; CS0 Wake-Up (to be set in PMU0FL)

; Friendly names for the LPM function arguments
PORT_MATCH   	SET PMATWK
RTC          	SET RTCFWK + RTCAWK
COMPARATOR   	SET CPT0WK

; FLSCL Bit Definition
BYPASS    	EQU 0x40
NON_ZERO  	EQU 0x01

;------------------------------------------------------------------------------
; End Of File
;------------------------------------------------------------------------------